module TB_instruction_decoder;
   reg clk;
   reg [11:0] inst;
   

   wire op_nop,
	op_movwf,
	op_clrw,
	op_clrf,
	op_subwf,    //  11       654   0
	op_decf,     //  opcode    d  f
			       
	op_andwf,
	op_xorwf,
	op_addwf,
	op_iorwf,
	op_movf,
	op_comf,
	op_incf,
	op_decfsz,
	op_rrf,
	op_rlf,
	op_swapf,
	op_incfsz,

	op_bcf,
	op_bsf,
	op_btfsc,
	op_btfss,

	op_option,
	op_sleep,
	op_clrwdt,

	op_tris,
	op_retlw,
	op_call,
	op_goto,
	op_movlw,
	op_iorlw,
	op_andlw,
	op_xorlw;
   
   wire [7:0] k;
   wire [8:0] longk;
   wire [4:0] fsel;
   wire     d, fwe, c_write, z_write;


   reg [11:0] mem[63:0];
   reg [5:0]  addr;
   
   instruction_decoder u_id(inst,
			    op_nop,
			    op_movwf,
			    op_clrw,
			    op_clrf,
			    op_subwf,    //  11       654   0
			    op_decf,     //  opcode    d  f
			       
			    op_andwf,
			    op_xorwf,
			    op_addwf,
			    op_iorwf,
			    op_movf,
			    op_comf,
			    op_incf,
			    op_decfsz,
			    op_rrf,
			    op_rlf,
			    op_swapf,
			    op_incfsz,
			    
			    op_bcf,
			    op_bsf,
			    op_btfsc,
			    op_btfss,

			    op_option,
			    op_sleep,
			    op_clrwdt,

			    op_tris,
			    op_retlw,
			    op_call,
			    op_goto,
			    op_movlw,
			    op_iorlw,
			    op_andlw,
			    op_xorlw
			    );

   // clock generator
   always begin
      #10 clk = 0;
      addr = addr+1;
      #10 clk = 1;
   end

   // reset
   initial begin
      addr =-1;
      $readmemh("tv_instruction_decoder.dat", mem);
      $display("test:%h", mem[0]);
      $display("instruction decoder testbench");
      $display("=============================");
   end

   always @(posedge clk) begin
      inst = mem[addr];
   end
   // observer
   always @(op_nop or 
	    op_movwf or 
	    op_clrw or 
	    op_clrf or 
	    op_subwf or     //  11       654   0
	    op_decf or      //  opcode    d  f
			       
	    op_andwf or 
	    op_xorwf or 
	    op_addwf or 
	    op_iorwf or 
	    op_movf or 
	    op_comf or 
	    op_incf or 
	    op_decfsz or 
	    op_rrf or 
	    op_rlf or 
	    op_swapf or 
	    op_incfsz or 
			    
	    op_bcf or 
	    op_bsf or 
	    op_btfsc or 
	    op_btfss or 
	    
	    op_option or 
	    op_sleep or 
	    op_clrwdt or 

	    op_tris or 
	    op_retlw or 
	    op_call or 
	    op_goto or 
	    op_movlw or 
	    op_iorlw or 
	    op_andlw or 
	    op_xorlw
	    )begin

      $display($time,,,
	       "%d:%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b%b ", addr, op_nop,
			       op_movwf,
			       op_clrw,
			       op_clrf,
			       op_subwf,    //  11       654   0
			       op_decf,     //  opcode    d  f
			       
			       op_andwf,
			       op_xorwf,
			       op_addwf,
			       op_iorwf,
			       op_movf,
			       op_comf,
			       op_incf,
			       op_decfsz,
			       op_rrf,
			       op_rlf,
			       op_swapf,
			       op_incfsz,

			       op_bcf,
			       op_bsf,
			       op_btfsc,
			       op_btfss,

			       op_option,
			       op_sleep,
			       op_clrwdt,

			       op_tris,
			       op_retlw,
			       op_call,
			       op_goto,
			       op_movlw,
			       op_iorlw,
			       op_andlw,
			       op_xorlw
	       );
      
   end // always @ (op_nop or ...
   
endmodule      