module TB_instruction_decoder_top;
   reg clk, reset, skip;
   reg [4:0] fsr_rs;
   reg [11:0] rom_data;

   wire op_nop,
	op_movwf,
	op_clrw,
	op_clrf,
	op_subwf,    //  11       654   0
	op_decf,     //  opcode    d  f
			       
	op_andwf,
	op_xorwf,
	op_addwf,
	op_iorwf,
	op_movf,
	op_comf,
	op_incf,
	op_decfsz,
	op_rrf,
	op_rlf,
	op_swapf,
	op_incfsz,

	op_bcf,
	op_bsf,
	op_btfsc,
	op_btfss,

	op_option,
	op_sleep,
	op_clrwdt,

	op_tris,
	op_retlw,
	op_call,
	op_goto,
	op_movlw,
	op_iorlw,
	op_andlw,
	op_xorlw;
   
   wire [7:0] k;
   wire [8:0] longk;
   wire [4:0] fsel;
   wire     d, fwe, c_write, z_write;


   reg [11:0] mem[2047:0];
   reg [12:0]  addr;
   
   instruction_decoder_top u_id(clk,
			    reset,
			    fsr_rs,
			    skip,
			    rom_data,
			    
			       
			    op_nop,
			    op_movwf,
			    op_clrw,
			    op_clrf,
			    op_subwf,    //  11       654   0
			    op_decf,     //  opcode    d  f
			       
			    op_andwf,
			    op_xorwf,
			    op_addwf,
			    op_iorwf,
			    op_movf,
			    op_comf,
			    op_incf,
			    op_decfsz,
			    op_rrf,
			    op_rlf,
			    op_swapf,
			    op_incfsz,
			    
			    op_bcf,
			    op_bsf,
			    op_btfsc,
			    op_btfss,

			    op_option,
			    op_sleep,
			    op_clrwdt,

			    op_tris,
			    op_retlw,
			    op_call,
			    op_goto,
			    op_movlw,
			    op_iorlw,
			    op_andlw,
			    op_xorlw,

			    k,
			    longk,
			    fsel,
			    d,
			    fwe,
			    c_write, 
			    z_write
			    );

   // clock generator
   always begin
      #10 clk = 0;
      addr = addr+1;
      #10 clk = 1;
   end

   // reset
   initial begin
      skip=0;
      reset=0;
      addr = -1;
      fsr_rs = 5'b00000;
      #2 reset = 1;
      #2 reset = 0;
      $readmemh("rom_1.dat", mem);
   end

   always @(posedge clk) begin
      rom_data = mem[addr];
   end
   
   // observer
   always @(negedge clk)begin

      $display($time,,,
	       "%d:%s %d %d %d %b %b %b %b", addr, 
	       (op_nop==1)? "nop" :
	       (op_movwf == 1) ? "movwf" :
	       (op_clrw == 1) ? "clrw" :
	       (op_clrf == 1) ? "clrf" :
	       (op_subwf==1) ? "subwf" :
	       (op_decf==1) ? "decf" :
	       
	       (op_andwf==1) ? "andwf":
	       (op_xorwf==1) ? "xorwf":
	       (op_addwf==1) ? "addwf":
	       (op_iorwf==1) ? "iorwf":
	       (op_movf==1) ? "movf":
	       (op_comf==1) ? "comf":
	       (op_incf==1) ? "incf":
	       (op_decfsz==1) ? "decfsz":
	       (op_rrf==1) ? "rrf":
	       (op_rlf==1) ? "rlf":
	       (op_swapf==1) ? "swapf":
	       (op_incfsz==1) ? "incfsz":
	       
	       (op_bcf==1) ? "bcf":
	       (op_bsf==1) ? "bsf":
	       (op_btfsc==1) ? "btfsc":
	       (op_btfss==1) ? "btfss":
	       
	       (op_option==1) ? "option":
	       (op_sleep==1) ? "sleep":
	       (op_clrwdt==1) ? "clrwdt":
	       
	       (op_tris==1) ? "tris":
	       (op_retlw==1) ? "retlw":
	       (op_call==1) ? "call":
	       (op_goto==1) ? "goto":
	       (op_movlw==1) ? "movlw":
	       (op_iorlw==1) ? "iorlw":
	       (op_andlw==1) ? "andlw":
	       (op_xorlw==1) ? "xorlw":"ERROR",

	       k,
	       longk,
	       fsel,
	       d,
	       fwe,
	       c_write, 
	       z_write);
   end // always @ (op_nop or ...
   
endmodule      