/*
 * PIC16C5x compatible processor
 * 
 * 
 *               Park, Chunghoon(shyfool@hanmail.net)
 *           High-Speed Digital Circuit Lab in Kyungpook National University
 * 
 * 
 * This is based on Ans_Risc_Core8(VHDL)(http://www.anslab.co.kr)
 * 
 * YOU CAN USE FREELY WITH YOUR OWN RISK
 * -------------------------------------
 */

module program_counter(clk,
		       reset,

		       aluz,
		       longk,
		       status_pa,
		       k,

		       fwe,
		       fsel,
		       fin,
	       
		       op_call,
		       op_goto,
		       op_retlw,
		       op_decfsz,
		       op_incfsz,
		       op_btfsc,
		       op_btfss,
		       
		       addr,
		       skip);

   input clk, reset, aluz;
   input [8:0] longk;
   input [7:0] k;
   input [1:0] status_pa;
   input       fwe;
   input [4:0] fsel;
   input [7:0] fin;
   input       op_call,
	       op_goto,
	       op_retlw,
	       op_decfsz,
	       op_incfsz,
	       op_btfsc,
	       op_btfss;
   
   
   output [10:0] addr;
   output 	 skip;


   reg 		 skip;
   reg [10:0]  pc;
   reg [1:0]  stacklevel;
   reg [10:0] stack1;
   reg [10:0] stack2;
   

   assign     addr = pc;
   
   always @(posedge clk or posedge reset) begin
      if(reset == 1) begin
	 stacklevel <= 2'b00;
	 stack1 <= 11'b0000_0000_000;
	 stack2 <= 11'b0000_0000_000;
	 pc <= 11'b111_1111_1110;
	 skip <= 0;
      end
      else begin
	 if(op_goto == 1) begin
	    pc[8:0] <= longk;
	    pc[10:9] <= status_pa;
	 end
	 else if(op_call == 1) begin
	    case(stacklevel) 
	      2'b00: begin
		 stack1 <= pc+1;
		 stacklevel <= 2'b01;
	      end
	      2'b01:  begin
		 stack2 <= pc+1;
		 stacklevel <= 2'b10;
	      end
	      default: begin
		 stack2 <= 2'b00;
		 stack1 <= 2'b00;
		 stacklevel <= 2'b00;
	      end
	    endcase // case(stacklevel)
	    pc[7:0] <= k;
	    pc[8] <= 0;
	    pc[10:9] <= status_pa;
	 end // if (op_call === 1)
	 else if(op_retlw == 1) begin
	    case (stacklevel)
	      2'b01: begin
		 pc <= stack1;
		 stacklevel <= 2'b00;
	      end
	      2'b10: begin
		 pc <= stack2;
		 stacklevel <=2'b01;
	      end
	    endcase // case(stacklevel)
	 end // if (op_retlw === 1)
	 else if((fwe == 1) && (fsel == 5'b00010)) begin
	    pc[7:0] <=fin;
	 end
	 else
	   pc <= pc+1;
	 
	 if((op_decfsz == 1 && aluz == 1) ||
	    (op_incfsz == 1 && aluz == 1) ||
	    (op_btfsc == 1 && aluz == 1) ||
	    (op_btfss == 1 && aluz == 1))
	   skip <= 1;
	 else
	   skip <= 0;
      end
   end   
endmodule // program_counter

   