/*
 * PIC16C5x compatible processor
 * 
 * 
 *               Park, Chunghoon(shyfool@hanmail.net)
 *           High-Speed Digital Circuit Lab in Kyungpook National University
 * 
 * 
 * This is based on Ans_Risc_Core8(VHDL)(http://www.anslab.co.kr)
 * 
 * YOU CAN USE FREELY WITH YOUR OWN RISK
 * -------------------------------------
 */

module reg_general(clk_q3,
		   clk_q4,
		   fwe,
		   fsel,
		   portA,
		   portB,
		   portC,
		   fsr,
		   status,
		   fin,
		   dout);
   input clk_q3,
	 clk_q4,
	 fwe;
   input [4:0] fsel;
   input [7:0] fin, portA, portB, portC, fsr, status;
   
   output [7:0] dout;
   
   reg [7:0] 	dout;
   
   
   reg [7:0] 	gpr[15:8],
		gpr0[31:16],gpr1[31:16], gpr2[31:16], gpr3[31:16];

   wire [1:0] 	bank_addr;
   wire [4:0] 	reg_addr;
   
   reg [7:0] 	reg_out;
   
   assign reg_addr = (fsel == 5'b00000) ? fsr[4:0]: fsel;
   assign bank_addr = fsr[6:5];

   // write to register and read
   always @(posedge clk_q4) begin
      if(fwe == 1) begin
	 if(reg_addr > 15) begin
	    case (bank_addr)
	      2'b00:
		gpr0[reg_addr] <= fin;
	      2'b01:
		gpr1[reg_addr] <= fin;
	      2'b10:
		gpr2[reg_addr] <= fin;
	      2'b11:
		gpr3[reg_addr] <= fin;
	    endcase // case(bank_addr)
	 end // if (reg_addr > 5'h10)
	 else if(reg_addr > 7) begin
	   gpr[reg_addr] <= fin;
	 end
      end // if (fwe == 1)
      reg_out <= fin;
   end // always @ (posedge clk_q4)
   
   // read from register
   always @(reg_addr or bank_addr or portA or portB or portC or status or fsr or reg_out)begin
      if(reg_addr > 5'h0F) begin
	 case(bank_addr)
	   2'b00:
	     dout <= gpr0[reg_addr];
	   2'b01:
	     dout <= gpr1[reg_addr];
	   2'b10:
	     dout <= gpr2[reg_addr];
	   2'b11:
	     dout <= gpr3[reg_addr];
	 endcase // case(bank_addr)
      end // if (reg_addr > 5'h0F)
      else if(reg_addr > 5'h07) begin
	 dout <= gpr[reg_addr];
      end
      else if(reg_addr == 7)
	dout <= portC;
      else if(reg_addr == 6)
	dout <= portB;
      else if(reg_addr == 5)
	dout <= portA;
      else if(reg_addr == 4)
	dout <= fsr;
      else if(reg_addr == 3)
	dout <= status;
      else
	dout <= reg_out;
   end
endmodule
